#include "MidiSynthesizer.hpp"
#include "constants.h"
#include "resource.h"
#include "StringTable.hpp"

MidiSynthesizer::MidiSynthesizer(HWND mainWindowHandle)
{
	UINT result = midiOutOpen(&midiOutHandle_, MIDIMAPPER, 0, 0, CALLBACK_NULL);
	if (result != MMSYSERR_NOERROR)
	{
		midiOutHandle_ = nullptr;
		if (result == MIDIERR_NODEVICE)
		{
			MessageBox(mainWindowHandle, StringTable::get(IDS_ERROR_OPEN_MIDI_OUT), StringTable::get(IDS_ERRORDIALOG_CAPTION), ERROR_DIALOG_TYPE);
		}
		else if (result == MMSYSERR_ALLOCATED)
		{
			MessageBox(mainWindowHandle, StringTable::get(IDS_ERROR_OPEN_MIDI_OUT), StringTable::get(IDS_ERRORDIALOG_CAPTION), ERROR_DIALOG_TYPE);
		}
		else if (result == MMSYSERR_BADDEVICEID)
		{
			MessageBox(mainWindowHandle, StringTable::get(IDS_ERROR_OPEN_MIDI_OUT), StringTable::get(IDS_ERRORDIALOG_CAPTION), ERROR_DIALOG_TYPE);
		}
		else if (result == MMSYSERR_INVALPARAM)
		{
			MessageBox(mainWindowHandle, StringTable::get(IDS_ERROR_OPEN_MIDI_OUT), StringTable::get(IDS_ERRORDIALOG_CAPTION), ERROR_DIALOG_TYPE);
		}
		else if (result == MMSYSERR_NOMEM)
		{
			MessageBox(mainWindowHandle, StringTable::get(IDS_ERROR_OPEN_MIDI_OUT), StringTable::get(IDS_ERRORDIALOG_CAPTION), ERROR_DIALOG_TYPE);
		}
	}
}

MidiSynthesizer::~MidiSynthesizer()
{
	if (midiOutHandle_ != nullptr)
	{
		midiOutReset(midiOutHandle_);
		midiOutClose(midiOutHandle_);
	}
}

void MidiSynthesizer::press(BYTE noteNumber)
{
	if (midiOutHandle_ != nullptr)
		midiOutShortMsg(midiOutHandle_, 0x007f0090 | (static_cast<DWORD>(noteNumber) << 8));
}

void MidiSynthesizer::release(BYTE noteNumber)
{
	if (midiOutHandle_ != nullptr)
		midiOutShortMsg(midiOutHandle_, 0x00000080 | (static_cast<DWORD>(noteNumber) << 8));
}

void MidiSynthesizer::reset()
{
	if (midiOutHandle_ != nullptr)
		midiOutReset(midiOutHandle_);
}
